"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceAll = exports.getApiConfig = exports.getStandaloneService = exports.getConfPath = exports.isStandalone = exports.adminCheck = exports.bypassAccessCheck = exports.accessCheck = exports.asyncWrapper = exports.getCoronerUrl = exports.getCoronerToken = void 0;
const express_1 = __importDefault(require("express"));
const backtrace_service_1 = require("backtrace-service");
const errors_1 = require("../lib/errors");
const body_parser_1 = require("body-parser");
const http_1 = __importDefault(require("http"));
const cors_1 = __importDefault(require("cors"));
const helmet_1 = __importDefault(require("helmet"));
const nocache_1 = __importDefault(require("nocache"));
const compression_1 = __importDefault(require("compression"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const coronerdUtils_1 = require("./coronerdUtils");
const index_1 = require("../model/query/index");
const filter_1 = require("../model/query/filter");
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const log = backtrace_service_1.Logger.getLogger();
const isStandalone = process.env.NODE_ENV === 'development' || process.env.NODE_ENV === 'test';
exports.isStandalone = isStandalone;
function getConfPath(serviceName) {
    let append = '';
    if (process.env.NODE_ENV === 'development') {
        append = '_dev';
    }
    else if (process.env.NODE_ENV === 'test') {
        append = '_test';
    }
    const confpath = path_1.default.join(__dirname, '..', '..', `${serviceName}${append}.conf`);
    return confpath;
}
exports.getConfPath = getConfPath;
function asyncWrapper(fn) {
    return (req, res, next) => {
        Promise.resolve(fn(req, res, next)).catch((e) => {
            if (isStandalone) {
                console.trace(e);
            }
            next(e);
        });
    };
}
exports.asyncWrapper = asyncWrapper;
async function getApiConfig(request, token, service, universe) {
    if (request.coronerAuth && request.coronerAuth.cfg) {
        return request.coronerAuth.cfg;
    }
    else {
        const coronerUrl = getCoronerUrl(universe, service, request);
        const configResponse = await coronerdUtils_1.fetchConfig(coronerUrl, token);
        return configResponse;
    }
}
exports.getApiConfig = getApiConfig;
function accessCheck(service) {
    return function accessCheck2(req, res, next) {
        if (isStandalone) {
            return next();
        }
        const projectName = req.query.project;
        const universeName = req.query.universe;
        if (!projectName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, 'missing param project name'));
        }
        if (!universeName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, 'missing param universe name'));
        }
        service.identityManager.checkProjectAccess(req, res, [projectName]);
        next();
    };
}
exports.accessCheck = accessCheck;
function bypassAccessCheck(service, allowProduction = false) {
    return (_req, res, next) => {
        if (isStandalone) {
            return next();
        }
        if (!allowProduction && !isStandalone) {
            log.error('Calling bypassAccessCheck in production.' +
                'Are you sure you want to do this? Pass true as a second parameter to override.');
            res.status(403).send();
        }
        service.identityManager.bypassProjectAccessCheck(res);
        next();
    };
}
exports.bypassAccessCheck = bypassAccessCheck;
function adminCheck(service) {
    return function adminCheck2(req, res, next) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (isStandalone) {
            return next();
        }
        const projectName = req.query.project;
        const universeName = req.query.universe;
        if (!projectName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, 'missing param project name'));
        }
        if (!universeName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, 'missing param universe name'));
        }
        const authRequest = req;
        const proj = (_d = (_c = (_b = (_a = authRequest === null || authRequest === void 0 ? void 0 : authRequest.coronerAuth) === null || _a === void 0 ? void 0 : _a.cfg) === null || _b === void 0 ? void 0 : _b.universes[universeName]) === null || _c === void 0 ? void 0 : _c.projects_ext) === null || _d === void 0 ? void 0 : _d.find((project) => project.name === projectName);
        const projRole = proj === null || proj === void 0 ? void 0 : proj.role;
        const userRole = (_g = (_f = (_e = authRequest === null || authRequest === void 0 ? void 0 : authRequest.coronerAuth) === null || _e === void 0 ? void 0 : _e.cfg) === null || _f === void 0 ? void 0 : _f.user) === null || _g === void 0 ? void 0 : _g.role;
        if (userRole === 'admin' || projRole === 'admin') {
            next();
        }
        else {
            return res.status(403).send();
        }
    };
}
exports.adminCheck = adminCheck;
function getCoronerToken(service, req) {
    var _a, _b;
    return ((_b = (_a = req === null || req === void 0 ? void 0 : req.coronerAuth) === null || _a === void 0 ? void 0 : _a.cfg) === null || _b === void 0 ? void 0 : _b.token) || process.env.CORONER_TOKEN || service.configuration.coronerToken;
}
exports.getCoronerToken = getCoronerToken;
function addTimestampRangeToBin(query, filters) {
    var _a, _b, _c, _d, _e;
    const timestampBin = (_b = (_a = query === null || query === void 0 ? void 0 : query.fold) === null || _a === void 0 ? void 0 : _a.timestamp) === null || _b === void 0 ? void 0 : _b.find((t) => t[0] === attributeAggregations_1.AggregationOp.BIN);
    if (timestampBin && timestampBin.length === 2) {
        const timestampRange = (_c = filters[0]) === null || _c === void 0 ? void 0 : _c.timestamp;
        const rangeStart = (_d = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_LEAST)) === null || _d === void 0 ? void 0 : _d[1];
        const rangeEnd = (_e = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_MOST)) === null || _e === void 0 ? void 0 : _e[1];
        if (rangeStart !== undefined && rangeEnd !== undefined) {
            const newTimestampBinAgg = [attributeAggregations_1.AggregationOp.BIN, timestampBin[1], rangeStart, rangeEnd];
            index_1.removeAggregation(query, 'timestamp', attributeAggregations_1.AggregationOp.BIN);
            index_1.addAggregation(query, 'timestamp', newTimestampBinAgg);
        }
    }
}
function replaceAll(initial, candidate, target) {
    let str = initial;
    while (str.includes(target)) {
        str = str.replace(target, candidate);
    }
    return str;
}
exports.replaceAll = replaceAll;
function getCoronerUrl(universeName, service, req) {
    var _a;
    return (((_a = req === null || req === void 0 ? void 0 : req.coronerAuth) === null || _a === void 0 ? void 0 : _a.url) ||
        service.configuration.coronerLocation ||
        process.env.CORONER_LOCATION ||
        `https://${universeName}.sp.backtrace.io`);
}
exports.getCoronerUrl = getCoronerUrl;
function getStandaloneService({ name, port, config = {} }) {
    const app = express_1.default();
    app.use(cors_1.default());
    app.use(nocache_1.default());
    app.use(helmet_1.default({
        hsts: false,
    }));
    app.use(body_parser_1.json());
    app.use(body_parser_1.urlencoded({ extended: true }));
    app.use(compression_1.default());
    let confFromFile;
    try {
        confFromFile = JSON.parse(fs_1.default.readFileSync(getConfPath(name)).toString());
    }
    catch (e) {
        log.error('Unable to parse service conf file: ');
        console.trace(e);
        confFromFile = {};
    }
    return {
        app,
        start: () => {
            const server = http_1.default.createServer(app);
            server.listen(port, () => {
                log.info('Service listening on port ' + port);
            });
        },
        configuration: Object.assign(config, confFromFile),
        apiPrefix: `api/${name}`,
        identityManager: {
            authenticateRequest: () => {
                return (_req, _res, next) => {
                    next();
                };
            },
        },
    };
}
exports.getStandaloneService = getStandaloneService;
//# sourceMappingURL=util.js.map